@@ -3,10 +3,20 @@
 test_years <- function(y.origin, y.dest){
    if ((y.origin - y.dest) == 0L)
      stop("ERROR: The origin and destination reference years must be different.")
-   if (!all(c(y.origin, y.dest) %in% c(2001L, 2003L:2022L)))
-     stop("ERROR: The origin and destination reference years allowed are 2001 and 2003 to 2022.")
+   if (!all(c(y.origin, y.dest) %in% c(2001L, 2003L:2023L)))
+     stop("ERROR: The origin and destination reference years allowed are 2001 and 2003 to 2023.")
 }
 
+# --------------------------------------
+# Deshace la reducción de tamanyo de las correspondencias entre sscc
+reverse_reduce_size <- function(lista){
+  temp <- mapply(function(a, b) data.frame(sscc = a, w = 1),
+                 lista[[1L]][, 2L], 1, SIMPLIFY = FALSE)
+  names(temp) <- lista[[1L]][, 1L]
+  lista <- append(temp, lista[[2L]])
+#  lista <- temp[order(names(temp))]
+  return(lista)
+}
 
 # --------------------------------------
 # Testea si los codigos de sscc son correctos y si existen
@@ -21,7 +31,7 @@ test_sscc_codes <- function(bbdd, y.origin, y.dest){
   codes.sc[l.codes == 9L] <- paste0("0", codes.sc[l.codes == 9L])
   nombre.lista <- paste0("sscc_", substr(y.origin, 3L, 4L), "_to_", substr(y.dest, 3L, 4L))
   lista.origen <- get(nombre.lista)
-  existen <- codes.sc %in% names(lista.origen)
+  existen <- codes.sc %in% c(lista.origen[[1L]][, 1L], names(lista.origen[[2L]]))
   sum.no.existen <- sum(!existen)
 
   if (sum.no.10 != 0L){
@@ -107,22 +117,23 @@ impute_total <- function(bbdd, y.origin, y.dest, all.units){
   nombre.lista <- paste0("sscc_", substr(y.dest, 3L, 4L), "_to_", substr(y.origin, 3L, 4L))
   lista.destino <- get(nombre.lista)
   variables <- names(bbdd)[-1L]
-  lista.origen <- lista.origen[names(lista.origen) %in% bbdd[, 1L]]
+  lista.origen1 <- lista.origen[[1L]][lista.origen[[1L]][, 1L] %in% bbdd[, 1L], ]
+  lista.origen2 <- lista.origen[[2L]][names(lista.origen[[2L]]) %in% bbdd[, 1L]]
 #  salida <- data.frame(names(lista.destino),
 #                       matrix(0L, length(lista.destino), length(variables)))
 #  names(salida) <- c("SSCC.Codes", variables)
   if (all.units){
-    salida <- data.frame(SSCC.Codes = names(lista.destino))
+    salida <- data.frame(SSCC.Codes = c(lista.destino[[1L]][, 1L], names(lista.destino[[2L]])))
   } else {
-    salida <- data.frame(SSCC.Codes = sort(unique(as.vector(unlist(sapply(lista.origen, function(x) x$sscc))))))
+    salida <- data.frame(SSCC.Codes = sort(unique(c(lista.origen1[, 2L],
+      as.vector(unlist(sapply(lista.origen2, function(x) x$sscc)))))))
   }
 
   # correspondencias unitarias
-  n.filas <- sapply(lista.origen, nrow)
-  n1 <- n.filas == 1L
+  n1 <- bbdd[, 1L] %in% lista.origen1[, 1L]
   if (sum(n1) > 0L){
-    sscc.destino <- data.frame("SSCC" = bbdd[n1, 1L],
-                               "SSCC.Codes" = unlist(sapply(lista.origen[n1], function(x) x$sscc)))
+    sscc.destino <- lista.origen1
+    names( sscc.destino) <- c("SSCC", "SSCC.Codes")
     temp <- merge(bbdd, sscc.destino, all.x = TRUE)
     temp[is.na(temp$SSCC.Codes), 2L:ncol(bbdd)] <- 0L
     temp <- temp[, -1L]
@@ -134,11 +145,11 @@ impute_total <- function(bbdd, y.origin, y.dest, all.units){
   }
 
   # correspondencias no unitarias
-  no.n1 <- n.filas != 1L
+  no.n1 <- !n1
   if (sum(no.n1) > 0L){
     bbdd.no <- bbdd[no.n1, ]
     for (rr in 1L:nrow(bbdd.no)){
-      destino <- lista.origen[[ which(names(lista.origen) == bbdd.no[rr, 1L]) ]]
+      destino <- lista.origen2[[ which(names(lista.origen2) == bbdd.no[rr, 1L]) ]]
       reparto <- as.matrix(destino$w) %*% as.matrix(bbdd.no[rr, -1L])
       for (ii in 1L:nrow(reparto)){
         fila <- which(destino$sscc[ii] == salida[, 1L])
@@ -163,21 +174,29 @@ impute_average <- function(bbdd, y.origin, y.dest, all.units){
   nombre.lista <- paste0("sscc_", substr(y.dest, 3L, 4L), "_to_", substr(y.origin, 3L, 4L))
   lista.destino <- get(nombre.lista)
   variables <- names(bbdd)[-1L]
-  lista.origen <- lista.origen[names(lista.origen) %in% bbdd[, 1L]]
+  lista.origen1 <- lista.origen[[1L]][lista.origen[[1L]][, 1L] %in% bbdd[, 1L], ]
+  lista.origen2 <- lista.origen[[2L]][names(lista.origen[[2L]]) %in% bbdd[, 1L]]
+
   if (all.units){
-    salida <- data.frame(SSCC.Codes = names(lista.destino))
+    salida <- data.frame(SSCC.Codes = c(lista.destino[[1L]][, 1L], names(lista.destino[[2L]])))
   } else {
-    salida <- data.frame(SSCC.Codes = sort(unique(as.vector(unlist(sapply(lista.origen, function(x) x$sscc))))))
+    salida <- data.frame(SSCC.Codes = sort(unique(c(lista.origen1[, 2L],
+                                                    as.vector(unlist(sapply(lista.origen2, function(x) x$sscc)))))))
   }
 
   # correspondencias unitarias
-  destinos <- sort(unique(as.vector(unlist(sapply(lista.origen, function(x) x$sscc)))))
-  n.filas <- sapply(lista.destino[names(lista.destino) %in% destinos], nrow)
-  n1 <- destinos[n.filas == 1L]
+  destinos <- sort(unique(c(lista.origen1[, 2L],
+                            as.vector(unlist(sapply(lista.origen2, function(x) x$sscc))))))
+  # n.filas <- sapply(lista.destino[names(lista.destino) %in% destinos], nrow)
+  # n1 <- destinos[n.filas == 1L]
+  n.filas <- lista.destino[[1L]][, 1L] %in% destinos
+  n1 <- lista.destino[[1L]][n.filas, 1L]
   temp <- data.frame("SSCC.Codes" = rep(NA, nrow(salida)))
   if(length(n1) > 0L){
-    sscc.destino <- data.frame("SSCC" = unlist(sapply(lista.destino[names(lista.destino) %in% n1],
-                                                      function(x) x$sscc)), "SSCC.Codes" = n1)
+    #sscc.destino <- data.frame("SSCC" = unlist(sapply(lista.destino[names(lista.destino) %in% n1],
+    #                                                  function(x) x$sscc)), "SSCC.Codes" = n1)
+    sscc.destino <- data.frame("SSCC" = lista.destino[[1L]][lista.destino[[1L]][, 1L] %in% n1, 2L],
+                               "SSCC.Codes" = n1)
     temp <- merge(bbdd, sscc.destino, all.y = TRUE)
     salida <- merge(salida, temp[, -1L], all.x = TRUE)
     salida[is.na(salida)] <- 0L
@@ -187,10 +206,12 @@ impute_average <- function(bbdd, y.origin, y.dest, all.units){
   }
 
   # correspondencias no unitarias
-  no.n1 <- destinos[n.filas != 1L]
+  # no.n1 <- destinos[n.filas != 1L]
+  no.n1 <- destinos[!(destinos %in% n1)]
+  lista.destino2 <- lista.destino[[2L]]
   if (length(no.n1) > 0L){
     for (rr in 1L:length(no.n1)){
-      donantes <- lista.destino[[ which(names(lista.destino) == no.n1[rr]) ]]
+      donantes <- lista.destino2[[ which(names(lista.destino2) == no.n1[rr]) ]]
       pesos <- donantes$w
       pesos.acum <- 0
       for (ii in 1L:length(pesos)){
